@@ -1,3 +1,20 @@
+/*
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
+
 package org.crosswire.sword.orb;
 
 import java.io.File;
@@ -27,7 +44,14 @@ public class SwordOrb extends Object implements HttpSessionBindingListener {
 	public static final String LEXDICTS = "Lexicons / Dictionaries";
 	public static final String GENBOOKS = "Generic Books";
 	public static final String DAILYDEVOS = "Daily Devotional";
-	static org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(new String[]{}, null);
+
+	static java.util.Properties p = new java.util.Properties();
+	static {
+		p.setProperty("com.sun.CORBA.codeset.charsets", "0x05010001, 0x00010109");    // UTF-8, UTF-16
+		p.setProperty("com.sun.CORBA.codeset.wcharsets", "0x00010109, 0x05010001");    // UTF-16, UTF-8
+	}
+
+	static org.omg.CORBA.ORB orb = org.omg.CORBA_2_3.ORB.init(new String[]{}, p);
 	static Hashtable clients = new Hashtable();
 	String ior = null;
 	String remoteAddr = null;
@@ -38,6 +62,11 @@ public class SwordOrb extends Object implements HttpSessionBindingListener {
 	long   blacklistTill = 0;
 
 	private SWMgr attach() {
+
+		// assert IOR has been set
+		if (ior == null)
+			return null;
+
 		SWMgr retVal = null;
 		try {
 System.out.println("attaching...");
@@ -47,7 +76,7 @@ System.out.println("calling testConnection");
 			retVal.testConnection();
 System.out.println("testConnection successful");
 		}
-		catch(Exception e) {
+		catch(Throwable e) {
 //			e.printStackTrace();
 			retVal = null;
 System.out.println("failed in attach");
@@ -64,6 +93,7 @@ System.out.println("failed in attach");
 	public void finalize () throws Throwable {
 		// shut down external process
 		try {
+System.out.println("calling finalize.");
 			getSWMgrInstance().terminate();
 		}
 		catch (Exception e) {}	// we know this doesn't return property cuz we killed the orb! :)
@@ -76,11 +106,14 @@ System.out.println("failed in attach");
 	public void valueUnbound(HttpSessionBindingEvent httpSessionBindingEvent) {
 		try {
 //			throw new Exception("value unbound; showing stacktrace");
-			getSWMgrInstance().terminate();
 			Vector orbs = (Vector)clients.get(remoteAddr);
+int size = -1;
 			if (orbs != null) {
-				orbs.remove(orb);
+size = orbs.size();
+				orbs.remove(this);
 			}
+System.out.println("calling valueUnbound. size before: " + size + "; size after: "+orbs.size());
+			getSWMgrInstance().terminate();
 		}
 		catch (Exception e) {}	// we know this doesn't return properly cuz we killed the orb! :)
 //		catch (Exception e) {e.printStackTrace();}	// we know this doesn't return properly cuz we killed the orb! :)
