@@ -1,3 +1,24 @@
+/******************************************************************************
+ * Simple example of how to retrieve an entry from a SWORD module
+ *
+ * $Id: lookup.cpp 2327 2009-04-22 11:42:33Z scribe $
+ *
+ * Copyright 1998-2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
+
 #include <stdio.h>
 #include <iostream>
 #include <stdlib.h>
@@ -10,6 +31,7 @@ using sword::MarkupFilterMgr;
 using sword::SWModule;
 using sword::FMT_WEBIF;
 using sword::FMT_HTMLHREF;
+using sword::FMT_RTF;
 using sword::ModMap;
 using sword::AttributeTypeList;
 using sword::AttributeList;
