@@ -1,3 +1,24 @@
+/******************************************************************************
+ * This simple example shows how to perform a search on a SWORD module
+ *
+ * $Id: search.cpp 2327 2009-04-22 11:42:33Z scribe $
+ *
+ * Copyright 1998-2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
+
 #include <stdio.h>
 #include <rawtext.h>
 #include <swmgr.h>
@@ -10,8 +31,17 @@
 using namespace sword;
 #endif
 
+  /*
+	 *			>=0 - regex
+	 *			-1  - phrase
+	 *			-2  - multiword
+	 *			-3  - entryAttrib (eg. Word//Lemma/G1234/)
+	 *			-4  - Lucene
+   */
+char SEARCH_TYPE=-2;
+
+char printed = 0;
 void percentUpdate(char percent, void *userData) {
-	static char printed = 0;
 	char maxHashes = *((char *)userData);
 	
 	while ((((float)percent)/100) * maxHashes > printed) {
@@ -33,11 +63,10 @@ int main(int argc, char **argv)
 	SWModule *target;
 	ListKey listkey;
 	ListKey scope;
-	VerseKey parser;
 	ModMap::iterator it;
 
-	if ((argc != 3) && (argc != 4)) {
-		fprintf(stderr, "\nusage: %s <modname> <\"search string\"> [\"search_scope\"]\n"
+	if ((argc < 3) || (argc > 5)) {
+		fprintf(stderr, "\nusage: %s <modname> <\"search string\"> [\"search_scope\"] [\"search again for string in previous result set\"]\n"
 							 "\tExample: search KJV \"swift hear slow speak\"\n\n", argv[0]);
 
 		exit(-1);
@@ -60,23 +89,29 @@ int main(int argc, char **argv)
 
 	target = (*it).second;
 
-	if (argc == 4) {			// if min / max specified
-		scope = parser.ParseVerseList(argv[3], parser, true);
+	if (argc > 3) {			// if min / max specified
+		SWKey *k = target->getKey();
+		VerseKey *parser = SWDYNAMIC_CAST(VerseKey, k);
+		VerseKey kjvParser;
+		if (!parser) parser = &kjvParser;	// use standard KJV parsing as fallback
+		scope = parser->ParseVerseList(argv[3], *parser, true);
 		scope.Persist(1);
 		target->setKey(scope);
 	}
 
 	std::cerr << "[0=================================50===============================100]\n ";
 	char lineLen = 70;
-  /*
-	 *			>=0 - regex
-	 *			-1  - phrase
-	 *			-2  - multiword
-	 *			-3  - entryAttrib (eg. Word//Lemma/G1234/)
-	 *			-4  - Lucene
-   */
-	listkey = target->Search(searchTerm.c_str(), -2, /*SEARCHFLAG_MATCHWHOLEENTRY*/ REG_ICASE, 0, 0, &percentUpdate, &lineLen);
-	std::cout << "\n";
+	listkey = target->search(searchTerm.c_str(), SEARCH_TYPE, /*SEARCHFLAG_MATCHWHOLEENTRY*/ REG_ICASE, 0, 0, &percentUpdate, &lineLen);
+	std::cerr << std::endl;
+	if (argc > 4) {			// if min / max specified
+		scope = listkey;
+		scope.Persist(1);
+		target->setKey(scope);
+		printed = 0;
+		std::cerr << " ";
+		listkey = target->search(argv[4], SEARCH_TYPE, /*SEARCHFLAG_MATCHWHOLEENTRY*/ REG_ICASE, 0, 0, &percentUpdate, &lineLen);
+		std::cerr << std::endl;
+	}
 	listkey.sort();
 	while (!listkey.Error()) {
 		std::cout << (const char *)listkey << std::endl;
