@@ -1,7 +1,7 @@
 /******************************************************************************
  *  utilstr.h	- prototypes for string utility functions
  *
- * $Id: utilstr.h 1864 2005-11-20 06:06:40Z scribe $
+ * $Id: utilstr.h 2391 2009-05-08 02:02:16Z chrislit $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -23,24 +23,17 @@
 #define UTILSTR_H
 
 #include <defs.h>
+#include <sysdata.h>
 
 SWORD_NAMESPACE_START
 
-/** Use stdstr to copy a const char* to a char*
-*/
-char *stdstr (char **iistr, const char *istr, unsigned int memPadFactor = 1);
-char *strstrip (char *istr);
-const char *stristr (const char *s1, const char *s2);
-int strnicmp(const char *s1, const char *s2, int len);
-int stricmp(const char *s1, const char *s2);
-// char *toupperstr(char *buf);
-// char *toupperstr_utf8(char *buf, unsigned int max = 0);
-
-/*
-extern "C" {
-int stricmp(const char *s1, const char *s2);
-}
+/** stdstr - clone a string
 */
+SWDLLEXPORT char *stdstr (char **iistr, const char *istr, unsigned int memPadFactor = 1);
+SWDLLEXPORT char *strstrip (char *istr);
+SWDLLEXPORT const char *stristr (const char *s1, const char *s2);
+SWDLLEXPORT int strnicmp(const char *s1, const char *s2, int len);
+SWDLLEXPORT int stricmp(const char *s1, const char *s2);
 
 /******************************************************************************
  * SW_toupper - array of uppercase values for any given Latin-1 value
@@ -50,5 +43,18 @@ int stricmp(const char *s1, const char *s2);
 extern const unsigned char SW_toupper_array[256];
 #define SW_toupper(c) SW_toupper_array[(unsigned char)c]
 
+/******************************************************************************
+ * getUniCharFromUTF8 - retrieves the next Unicode codepoint from a UTF8 string
+ * 					and increments buf to start of next codepoint
+ *
+ * ENT:	buf - address of a utf8 buffer
+ *
+ * RET:	buf - incremented past last byte used in computing the current codepoint
+ * 		unicode codepoint value (0 with buf incremented is invalid UTF8 byte
+ */
+
+__u32 getUniCharFromUTF8(const unsigned char **buf);
+
+
 SWORD_NAMESPACE_END
 #endif
