@@ -1,7 +1,25 @@
  /*****************************************************************************
  * FTPTransport functions
  *
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
  */
+
+
  
 
 #include <ftptrans.h>
@@ -11,6 +29,9 @@
 #include <dirent.h>
 #include <swlog.h>
 
+extern "C" {
+#include <ftpparse.h>
+}
 
 using std::vector;
 
@@ -40,6 +61,8 @@ void StatusReporter::statusUpdate(double dtTotal, double dlNow) {
 FTPTransport::FTPTransport(const char *host, StatusReporter *statusReporter) {
 	this->statusReporter = statusReporter;
 	this->host = host;
+	u = "ftp";
+	p = "installmgr@user.com";
 	term = false;
 }
 
