@@ -2,9 +2,26 @@
  * InstallMgr functions to be made into something usefully exposed by
  * master Glassey
  *
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
  */
 
 
+
 #ifndef EXCLUDEZLIB
 extern "C" {
 #include <untgz.h>
@@ -18,19 +35,46 @@ extern "C" {
 #include <fcntl.h>
 
 #include <swmgr.h>
+#include <swmodule.h>
+#include <swversion.h>
+#include <swlog.h>
 #include <dirent.h>
 
 #include <stdio.h>
+#include <map>
 
 #ifdef CURLAVAILABLE
 #include <curlftpt.h>
+#include <curlhttpt.h>
 #else
 #include <ftplibftpt.h>
 #endif
 
-
+#include <iostream>
 SWORD_NAMESPACE_START
 
+namespace {
+
+void removeTrailingSlash(SWBuf &buf) {
+	int len = buf.size();
+	if ((buf[len-1] == '/')
+	 || (buf[len-1] == '\\'))
+		buf.size(len-1);
+}
+
+const char *masterRepoList = "masterRepoList.conf";
+
+};
+
+
+using std::map;
+
+const int InstallMgr::MODSTAT_OLDER            = 0x001;
+const int InstallMgr::MODSTAT_SAMEVERSION      = 0x002;
+const int InstallMgr::MODSTAT_UPDATED          = 0x004;
+const int InstallMgr::MODSTAT_NEW              = 0x008;
+const int InstallMgr::MODSTAT_CIPHERED         = 0x010;
+const int InstallMgr::MODSTAT_CIPHERKEYPRESENT = 0x020;
 
 // override this method and provide your own custom FTPTransport subclass
 // here we try a couple defaults if sword was compiled with support for them.
@@ -43,47 +87,84 @@ FTPTransport *InstallMgr::createFTPTransport(const char *host, StatusReporter *s
 #endif
 }
 
+FTPTransport *InstallMgr::createHTTPTransport(const char *host, StatusReporter *statusReporter) {
+#ifdef CURLAVAILABLE
+	return new CURLHTTPTransport(host, statusReporter);
+#else
+	return 0;
+#endif
+}
 
 
 
-InstallMgr::InstallMgr(const char *privatePath, StatusReporter *sr) {
+InstallMgr::InstallMgr(const char *privatePath, StatusReporter *sr, SWBuf u, SWBuf p) {
+	userDisclaimerConfirmed = false;
 	statusReporter = sr;
+	this->u = u;
+	this->p = p;
 	this->privatePath = 0;
 	this->transport = 0;
+	installConf = 0;
 	stdstr(&(this->privatePath), privatePath);
-	SWBuf confPath = (SWBuf)privatePath + "/InstallMgr.conf";
+	if (this->privatePath) {
+		int len = strlen(this->privatePath);
+		if ((this->privatePath[len-1] == '/')
+		 || (this->privatePath[len-1] == '\\'))
+			this->privatePath[len-1] = 0;
+	}
+	confPath = (SWBuf)privatePath + "/InstallMgr.conf";
 	FileMgr::createParent(confPath.c_str());
 	
+	readInstallConf();
+}
+
+
+InstallMgr::~InstallMgr() {
+	delete [] privatePath;
+	delete installConf;
+	clearSources();
+}
+
+void InstallMgr::clearSources() {
+	for (InstallSourceMap::iterator it = sources.begin(); it != sources.end(); ++it) {
+		delete it->second;
+	}
+	sources.clear();
+}
+
+void InstallMgr::readInstallConf() {
+
+	if (installConf) delete installConf;
+
 	installConf = new SWConfig(confPath.c_str());
 
-	SectionMap::iterator sourcesSection;
+	clearSources();
+	
+	setFTPPassive(stricmp((*installConf)["General"]["PassiveFTP"].c_str(), "false") != 0);
+
+	SectionMap::iterator confSection = installConf->Sections.find("Sources");
 	ConfigEntMap::iterator sourceBegin;
 	ConfigEntMap::iterator sourceEnd;
 
-	sources.clear();
-	
-	setFTPPassive(stricmp((*installConf)["General"]["PassiveFTP"].c_str(), "false")!=0);
-
-	sourcesSection = installConf->Sections.find("Sources");
-	if (sourcesSection != installConf->Sections.end()) {
-		sourceBegin = sourcesSection->second.lower_bound("FTPSource");
-		sourceEnd = sourcesSection->second.upper_bound("FTPSource");
+	if (confSection != installConf->Sections.end()) {
+		sourceBegin = confSection->second.lower_bound("FTPSource");
+		sourceEnd = confSection->second.upper_bound("FTPSource");
 
 		while (sourceBegin != sourceEnd) {
 			InstallSource *is = new InstallSource("FTP", sourceBegin->second.c_str());
 			sources[is->caption] = is;
-			SWBuf parent = (SWBuf)privatePath + "/" + is->source + "/file";
+			SWBuf parent = (SWBuf)privatePath + "/" + is->uid + "/file";
 			FileMgr::createParent(parent.c_str());
-			is->localShadow = (SWBuf)privatePath + "/" + is->source;
+			is->localShadow = (SWBuf)privatePath + "/" + is->uid;
 			sourceBegin++;
 		}
 	}
 
 	defaultMods.clear();
-	sourcesSection = installConf->Sections.find("General");
-	if (sourcesSection != installConf->Sections.end()) {
-		sourceBegin = sourcesSection->second.lower_bound("DefaultMod");
-		sourceEnd = sourcesSection->second.upper_bound("DefaultMod");
+	confSection = installConf->Sections.find("General");
+	if (confSection != installConf->Sections.end()) {
+		sourceBegin = confSection->second.lower_bound("DefaultMod");
+		sourceEnd = confSection->second.upper_bound("DefaultMod");
 
 		while (sourceBegin != sourceEnd) {
 			defaultMods.insert(sourceBegin->second.c_str());
@@ -93,18 +174,24 @@ InstallMgr::InstallMgr(const char *privatePath, StatusReporter *sr) {
 }
 
 
-InstallMgr::~InstallMgr() {
-	delete [] privatePath;
-	delete installConf;
+void InstallMgr::saveInstallConf() {
+
+	installConf->Sections["Sources"].erase("FTPSource");
 
 	for (InstallSourceMap::iterator it = sources.begin(); it != sources.end(); ++it) {
-		delete it->second;
+		if (it->second) {
+			installConf->Sections["Sources"].insert(ConfigEntMap::value_type("FTPSource", it->second->getConfEnt().c_str()));
+		}
 	}
+	(*installConf)["General"]["PassiveFTP"] = (isFTPPassive()) ? "true" : "false";
+
+	installConf->Save();
 }
 
 
 void InstallMgr::terminate() { if (transport) transport->terminate(); }
 
+
 int InstallMgr::removeModule(SWMgr *manager, const char *moduleName) {
 	SectionMap::iterator module;
 	ConfigEntMap::iterator fileBegin;
@@ -119,7 +206,7 @@ int InstallMgr::removeModule(SWMgr *manager, const char *moduleName) {
 		// to be sure all files are closed
 		// this does not remove the .conf information from SWMgr
 		manager->deleteModule(modName);
-			
+
 		fileBegin = module->second.lower_bound("File");
 		fileEnd = module->second.upper_bound("File");
 
@@ -127,6 +214,7 @@ int InstallMgr::removeModule(SWMgr *manager, const char *moduleName) {
 		SWBuf modDir;
 		entry = module->second.find("AbsoluteDataPath");
 		modDir = entry->second.c_str();
+		removeTrailingSlash(modDir);
 		if (fileBegin != fileEnd) {	// remove each file
 			while (fileBegin != fileEnd) {
 				modFile = modDir;
@@ -145,11 +233,12 @@ int InstallMgr::removeModule(SWMgr *manager, const char *moduleName) {
 
 			FileMgr::removeDir(modDir.c_str());
 
-			if (dir = opendir(manager->configPath)) {	// find and remove .conf file
+			if ((dir = opendir(manager->configPath))) {	// find and remove .conf file
 				rewinddir(dir);
 				while ((ent = readdir(dir))) {
 					if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
 						modFile = manager->configPath;
+						removeTrailingSlash(modFile);
 						modFile += "/";
 						modFile += ent->d_name;
 						SWConfig *config = new SWConfig(modFile.c_str());
@@ -169,64 +258,38 @@ int InstallMgr::removeModule(SWMgr *manager, const char *moduleName) {
 }
 
 
-
-InstallSource::InstallSource(const char *type, const char *confEnt) {
-	this->type = type;
-	mgr = 0;
-	userData = 0;
-	if (confEnt) {
-		char *buf = 0;
-		stdstr(&buf, confEnt);
-
-		caption = strtok(buf, "|");
-		source = strtok(0, "|");
-		directory = strtok(0, "|");
-		delete [] buf;
-	}
-}
-
-
-InstallSource::~InstallSource() {
-	if (mgr)
-		delete mgr;
-}
-
-
-void InstallSource::flush() {
-	if (mgr) {
-		delete mgr;
-		mgr = 0;
-	}
-}
-
-
-SWMgr *InstallSource::getMgr() {
-	if (!mgr)
-		// ..., false = don't augment ~home directory.
-		mgr = new SWMgr(localShadow.c_str(), true, 0, false, false);
-	return mgr;
-}
-
-
 int InstallMgr::ftpCopy(InstallSource *is, const char *src, const char *dest, bool dirTransfer, const char *suffix) {
+
+	// assert user disclaimer has been confirmed
+	if (!isUserDisclaimerConfirmed()) return -1;
+
 	int retVal = 0;
-	long i;
 	FTPTransport *trans = createFTPTransport(is->source, statusReporter);
 	transport = trans; // set classwide current transport for other thread terminate() call
+	if (is->u.length()) {
+		trans->setUser(is->u);
+		trans->setPasswd(is->p);
+	}
+	else {
+		trans->setUser(u);
+		trans->setPasswd(p);
+	}
 	trans->setPassive(passive);
-	
+
 	SWBuf urlPrefix = (SWBuf)"ftp://" + is->source;
 
 	// let's be sure we can connect.  This seems to be necessary but sucks
 //	SWBuf url = urlPrefix + is->directory.c_str() + "/"; //dont forget the final slash
-//	if (trans->getURL("dirlist", url.c_str())) {
-//		 fprintf(stderr, "FTPCopy: failed to get dir %s\n", url.c_str());
+//	if (trans->getURL("swdirlist.tmp", url.c_str())) {
+//		 SWLog::getSystemLog()->logDebug("FTPCopy: failed to get dir %s\n", url.c_str());
 //		 return -1;
 //	}
 
-	   
+
 	if (dirTransfer) {
-		SWBuf dir = (SWBuf)is->directory.c_str() + "/" + src; //dont forget the final slash
+		SWBuf dir = (SWBuf)is->directory.c_str();
+		removeTrailingSlash(dir);
+		dir += (SWBuf)"/" + src; //dont forget the final slash
 
 		retVal = trans->copyDirectory(urlPrefix, dir, dest, suffix);
 
@@ -234,9 +297,11 @@ int InstallMgr::ftpCopy(InstallSource *is, const char *src, const char *dest, bo
 	}
 	else {
 		SWTRY {
-			SWBuf url = urlPrefix + is->directory.c_str() + "/" + src; //dont forget the final slash
+			SWBuf url = urlPrefix + is->directory.c_str();
+			removeTrailingSlash(url);
+			url += (SWBuf)"/" + src; //dont forget the final slash
 			if (trans->getURL(dest, url.c_str())) {
-				fprintf(stderr, "FTPCopy: failed to get file %s", url.c_str());
+				SWLog::getSystemLog()->logDebug("FTPCopy: failed to get file %s", url.c_str());
 				retVal = -1;
 			}
 		}
@@ -269,13 +334,17 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 	struct dirent *ent;
 	SWBuf modFile;
 
+	SWLog::getSystemLog()->logDebug("***** InstallMgr::installModule\n");
+	if (fromLocation)
+		SWLog::getSystemLog()->logDebug("***** fromLocation: %s \n", fromLocation);
+	SWLog::getSystemLog()->logDebug("***** modName: %s \n", modName);
 
 	if (is)
-		sourceDir = (SWBuf)privatePath + "/" + is->source;
+		sourceDir = (SWBuf)privatePath + "/" + is->uid;
 	else	sourceDir = fromLocation;
 
-	if (sourceDir[sourceDir.length()-1] != '/')
-		sourceDir += '/';
+	removeTrailingSlash(sourceDir);
+	sourceDir += '/';
 
 	SWMgr mgr(sourceDir.c_str());
 	
@@ -297,7 +366,7 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 		if (fileBegin != fileEnd) {	// copy each file
 			if (is) {
 				while (fileBegin != fileEnd) {	// ftp each file first
-					buffer = sourceDir + "/" + fileBegin->second.c_str();
+					buffer = sourceDir + fileBegin->second.c_str();
 					if (ftpCopy(is, fileBegin->second.c_str(), buffer.c_str())) {
 						aborted = true;
 						break;	// user aborted
@@ -313,8 +382,8 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 					SWBuf sourcePath = sourceDir;
 					sourcePath += fileBegin->second.c_str();
 					SWBuf dest = destMgr->prefixPath;
-					if ((destMgr->prefixPath[strlen(destMgr->prefixPath)-1] != '\\') && (destMgr->prefixPath[strlen(destMgr->prefixPath)-1] != '/'))
-						dest += "/";
+					removeTrailingSlash(dest);
+					dest += '/';
 					dest += fileBegin->second.c_str();
 					FileMgr::copyFile(sourcePath.c_str(), dest.c_str());
 
@@ -326,7 +395,7 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 			if (is) {
 				fileBegin = module->second.lower_bound("File");
 				while (fileBegin != fileEnd) {	// delete each tmp ftp file
-					buffer = sourceDir + "/" + fileBegin->second.c_str();
+					buffer = sourceDir + fileBegin->second.c_str();
 					FileMgr::removeFile(buffer.c_str());
 					fileBegin++;
 				}
@@ -345,7 +414,17 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 			if (entry != module->second.end()) {
 				SWBuf absolutePath = entry->second.c_str();
 				SWBuf relativePath = absolutePath;
-				relativePath << strlen(mgr.prefixPath);
+				entry = module->second.find("PrefixPath");
+				if (entry != module->second.end()) {
+					relativePath << strlen(entry->second.c_str());
+				}
+				else {
+					relativePath << strlen(mgr.prefixPath);
+				}
+				SWLog::getSystemLog()->logDebug("***** mgr.prefixPath: %s \n", mgr.prefixPath);
+				SWLog::getSystemLog()->logDebug("***** destMgr->prefixPath: %s \n", destMgr->prefixPath);
+				SWLog::getSystemLog()->logDebug("***** absolutePath: %s \n", absolutePath.c_str());
+				SWLog::getSystemLog()->logDebug("***** relativePath: %s \n", relativePath.c_str());
 
 				if (is) {
 					if (ftpCopy(is, relativePath.c_str(), absolutePath.c_str(), true)) {
@@ -363,8 +442,8 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 			}
 		}
 		if (!aborted) {
-			SWBuf confDir = sourceDir + "/mods.d/";
-			if (dir = opendir(confDir.c_str())) {	// find and copy .conf file
+			SWBuf confDir = sourceDir + "mods.d/";
+			if ((dir = opendir(confDir.c_str()))) {	// find and copy .conf file
 				rewinddir(dir);
 				while ((ent = readdir(dir))) {
 					if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
@@ -373,6 +452,7 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 						SWConfig *config = new SWConfig(modFile.c_str());
 						if (config->Sections.find(modName) != config->Sections.end()) {
 							SWBuf targetFile = destMgr->configPath; //"./mods.d/";
+							removeTrailingSlash(targetFile);
 							targetFile += "/";
 							targetFile += ent->d_name;
 							FileMgr::copyFile(modFile.c_str(), targetFile.c_str());
@@ -399,52 +479,16 @@ int InstallMgr::installModule(SWMgr *destMgr, const char *fromLocation, const ch
 	return 1;
 }
 
-
-// override this and provide an input mechanism to allow your users
-// to enter the decipher code for a module.
-// return true you added the cipher code to the config.
-// default to return 'aborted'
-bool InstallMgr::getCipherCode(const char *modName, SWConfig *config) {
-	return false;
-
-/* a sample implementation, roughly taken from the windows installmgr
-
-	SectionMap::iterator section;
-	ConfigEntMap::iterator entry;
-	SWBuf tmpBuf;
-	section = config->Sections.find(modName);
-	if (section != config->Sections.end()) {
-		entry = section->second.find("CipherKey");
-		if (entry != section->second.end()) {
-			entry->second = GET_USER_INPUT();
-			config->Save();
-
-			// LET'S SHOW THE USER SOME SAMPLE TEXT FROM THE MODULE
-			SWMgr *mgr = new SWMgr();
-			SWModule *mod = mgr->Modules[modName];
-			mod->setKey("Ipet 2:12");
-			tmpBuf = mod->StripText();
-			mod->setKey("gen 1:10");
-			tmpBuf += "\n\n";
-			tmpBuf += mod->StripText();
-			SOME_DIALOG_CONTROL->SETTEXT(tmpBuf.c_str());
-			delete mgr;
-
-			// if USER CLICKS OK means we should return true
-			return true;
-		}
-	}
-	return false;
-*/
-
-}
-
-
 int InstallMgr::refreshRemoteSource(InstallSource *is) {
-	SWBuf root = (SWBuf)privatePath + (SWBuf)"/" + is->source.c_str();
+
+	// assert user disclaimer has been confirmed
+	if (!isUserDisclaimerConfirmed()) return -1;
+
+	SWBuf root = (SWBuf)privatePath + (SWBuf)"/" + is->uid.c_str();
+	removeTrailingSlash(root);
 	SWBuf target = root + "/mods.d";
 	int errorCode = -1; //0 means successful
-	
+
 	FileMgr::removeDir(target.c_str());
 
 	if (!FileMgr::existsDir(target))
@@ -452,17 +496,17 @@ int InstallMgr::refreshRemoteSource(InstallSource *is) {
 
 #ifndef EXCLUDEZLIB
 	SWBuf archive = root + "/mods.d.tar.gz";
-	
+
 	errorCode = ftpCopy(is, "mods.d.tar.gz", archive.c_str(), false);
 	if (!errorCode) { //sucessfully downloaded the tar,gz of module configs
 		FileDesc *fd = FileMgr::getSystemFileMgr()->open(archive.c_str(), FileMgr::RDONLY);
 		untargz(fd->getFd(), root.c_str());
 		FileMgr::getSystemFileMgr()->close(fd);
 	}
-	else if (!term) //if the tar.gz download was canceled don't continue with another download
+	else
 #endif
 	errorCode = ftpCopy(is, "mods.d", target.c_str(), true, ".conf"); //copy the whole directory
-	
+
 	is->flush();
 	return errorCode;
 }
@@ -472,6 +516,174 @@ bool InstallMgr::isDefaultModule(const char *modName) {
 	return defaultMods.count(modName);
 }
 
+/************************************************************************
+ * getModuleStatus - compare the modules of two SWMgrs and return a 
+ * 	vector describing the status of each.  See MODSTAT_*
+ */
+map<SWModule *, int> InstallMgr::getModuleStatus(const SWMgr &base, const SWMgr &other) {
+	map<SWModule *, int> retVal;
+	SWBuf targetVersion;
+	SWBuf sourceVersion;
+	SWBuf softwareVersion;
+	bool cipher;
+	bool keyPresent;
+	int modStat;
+	
+	for (ModMap::const_iterator mod = other.Modules.begin(); mod != other.Modules.end(); mod++) {
+	
+		modStat = 0;
+
+		cipher = false;
+		keyPresent = false;
+		
+		const char *v = mod->second->getConfigEntry("CipherKey");
+		if (v) {
+			cipher = true;
+			keyPresent = *v;
+		}
+		
+		targetVersion = "0.0";
+		sourceVersion = "1.0";
+		softwareVersion = (const char *)SWVersion::currentVersion;
+		
+		v = mod->second->getConfigEntry("Version");
+		if (v) sourceVersion = v;
+
+		v = mod->second->getConfigEntry("MinimumVersion");
+		if (v) softwareVersion = v;
+
+		const SWModule *baseMod = base.getModule(mod->first);
+		if (baseMod) {
+			targetVersion = "1.0";
+			v = baseMod->getConfigEntry("Version");
+			if (v) targetVersion = v;
+			modStat |= (SWVersion(sourceVersion.c_str()) > SWVersion(targetVersion.c_str())) ? MODSTAT_UPDATED : (SWVersion(sourceVersion.c_str()) < SWVersion(targetVersion.c_str())) ? MODSTAT_OLDER : MODSTAT_SAMEVERSION;
+		}
+		else modStat |= MODSTAT_NEW;
+
+		if (cipher) modStat |= MODSTAT_CIPHERED;
+		if (keyPresent) modStat |= MODSTAT_CIPHERKEYPRESENT;
+		retVal[mod->second] = modStat;
+	}
+	return retVal;
+}
+
+
+/************************************************************************
+ * refreshRemoteSourceConfiguration - grab master list of know remote
+ * 	sources and integrate it with our configurations.
+ */
+int InstallMgr::refreshRemoteSourceConfiguration() {
+
+	// assert user disclaimer has been confirmed
+	if (!isUserDisclaimerConfirmed()) return -1;
+
+	SWBuf root = (SWBuf)privatePath;
+	removeTrailingSlash(root);
+	SWBuf masterRepoListPath = root + "/" + masterRepoList;
+	InstallSource is("FTP");
+	is.source = "ftp.crosswire.org";
+	is.directory = "/pub/sword";
+	int errorCode = ftpCopy(&is, masterRepoList, masterRepoListPath.c_str(), false);
+	if (!errorCode) { //sucessfully downloaded the repo list
+		SWConfig masterList(masterRepoListPath);
+		SectionMap::iterator sections = masterList.Sections.find("Repos");
+		if (sections != masterList.Sections.end()) {
+			for (ConfigEntMap::iterator actions = sections->second.begin(); actions != sections->second.end(); actions++) {
+				// Search through our current sources and see if we have a matching UID
+				InstallSourceMap::iterator it;
+				for (it = sources.begin(); it != sources.end(); ++it) {
+					// is this our UID?
+					if ((it->second) && (it->second->uid == actions->first)) {
+						if (actions->second == "REMOVE") {
+							// be sure to call save/reload after this
+							// or this could be dangerous
+							delete it->second;
+							it->second = 0;
+						}
+						else {
+							SWBuf key = actions->second.stripPrefix('=');
+							if (key == "FTPSource") {
+								// we might consider instantiating a temp IS
+								// from our config string and then copy only
+								// some entries.  This would allow the use to
+								// change some fields and not have them overwritten
+								// but it seems like we might want to change any
+								// of the current fields so we don't do this now
+								// InstallSource i("FTP", actions->second);
+								delete it->second;
+								it->second = new InstallSource("FTP", actions->second.c_str());
+								it->second->uid = actions->first;
+							}
+						}
+						break;
+					}
+				}
+				// didn't find our UID, let's add it
+				if (it == sources.end()) {
+					SWBuf key = actions->second.stripPrefix('=');
+					if (key == "FTPSource") {
+						if (actions->second != "REMOVE") {
+							InstallSource *is = new InstallSource("FTP", actions->second.c_str());
+							is->uid = actions->first;
+							sources[is->caption] = is;
+						}
+					}
+				}
+			}
+
+			// persist and re-read
+			saveInstallConf();
+			readInstallConf();
+
+			return 0;
+		}
+	}
+	return -1;
+}
+
+
+InstallSource::InstallSource(const char *type, const char *confEnt) {
+	this->type = type;
+	mgr = 0;
+	userData = 0;
+	if (confEnt) {
+		SWBuf buf = confEnt;
+		caption   = buf.stripPrefix('|', true);
+		source    = buf.stripPrefix('|', true);
+		directory = buf.stripPrefix('|', true);
+		u         = buf.stripPrefix('|', true);
+		p         = buf.stripPrefix('|', true);
+		uid       = buf.stripPrefix('|', true);
+
+		if (!uid.length()) uid = source;
+
+		removeTrailingSlash(directory);
+	}
+}
+
+
+InstallSource::~InstallSource() {
+	if (mgr)
+		delete mgr;
+}
+
+
+void InstallSource::flush() {
+	if (mgr) {
+		delete mgr;
+		mgr = 0;
+	}
+}
+
+
+SWMgr *InstallSource::getMgr() {
+	if (!mgr)
+		// ..., false = don't augment ~home directory.
+		mgr = new SWMgr(localShadow.c_str(), true, 0, false, false);
+	return mgr;
+}
+
 
 SWORD_NAMESPACE_END
 
