@@ -1,17 +1,25 @@
 /***************************************************************************
-                     osisrtf.cpp  -  OSIS to RTF filter
-                             -------------------
-    begin                : 2003-02-15
-    copyright            : 2003 by CrossWire Bible Society
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation version 2 of the License.                *
- *                                                                         *
- ***************************************************************************/
+ *                     osisrtf.cpp  -  OSIS to RTF filter
+ *                            -------------------
+ *   begin                : 2003-02-15
+ *   copyright            : 2003 by CrossWire Bible Society
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -272,6 +280,21 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 			}
 		}
 
+		// Milestoned paragraphs, created by osis2mod
+		// <div type="paragraph" sID.../>
+		// <div type="paragraph" eID.../>
+		else if (tag.isEmpty() && !strcmp(tag.getName(), "div") && tag.getAttribute("type") && !strcmp(tag.getAttribute("type"), "paragraph")) {
+			// <div type="paragraph"  sID... />
+			if (tag.getAttribute("sID")) {	// non-empty start tag
+				outText("{\\fi200\\par}", buf, u);
+			}
+			// <div type="paragraph"  eID... />
+			else if (tag.getAttribute("eID")) {
+				outText("{\\par}", buf, u);
+				userData->supressAdjacentWhitespace = true;
+			}
+		}
+
 		// <reference> tag
 		else if (!strcmp(tag.getName(), "reference")) {
 			if (!u->inXRefNote) {	// only show these if we're not in an xref note
@@ -316,6 +339,25 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 				outText("\\par}", buf, u);
 			}
 		}
+		// <list>	 - how do we support these better in RTF?
+		else if (!strcmp(tag.getName(), "list")) {
+			if((!tag.isEndTag()) && (!tag.isEmpty())) {
+				outText("\\par\\pard", buf, u);
+			}
+			else if (tag.isEndTag()) {
+				outText("\\par\\pard", buf, u);
+			}
+		}
+
+		// <item> - support better
+		else if (!strcmp(tag.getName(), "item")) {
+			if((!tag.isEndTag()) && (!tag.isEmpty())) {
+				outText("* ", buf, u);
+			}
+			else if (tag.isEndTag()) {
+				outText("\\par", buf, u);
+			}
+		}
 
 		// <catchWord> & <rdg> tags (italicize)
 		else if (!strcmp(tag.getName(), "rdg") || !strcmp(tag.getName(), "catchWord")) {
@@ -445,16 +487,10 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 		else if (!strcmp(tag.getName(), "divineName")) {
 
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				u->suspendTextPassThru = (++u->suspendLevel);
+ 				outText("{\\scaps ", buf, u);
 			}
 			else if (tag.isEndTag()) {
-				SWBuf lastText = u->lastSuspendSegment.c_str();
-				u->suspendTextPassThru = (--u->suspendLevel);
-				if (lastText.size()) {
-					toupperstr(lastText);
-					scratch.setFormatted("{\\fs19%c\\fs16%s}", lastText[0], lastText.c_str()+1);
-					outText(scratch.c_str(), buf, u);
-				}               
+				outText("}", buf, u);
 			}
 		}
 
@@ -462,10 +498,11 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 		else if (!strcmp(tag.getName(), "div")) {
 
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				outText("\\par\\par\\pard ", buf, u);
+				outText("\\pard ", buf, u);
 			}
 			else if (tag.isEndTag()) {
-			}
+	                        outText("\\par ", buf, u);
+                        }
 		}
 
 		// image
