@@ -1,18 +1,24 @@
 /***************************************************************************
-                     teirtf.cpp  -  TEI to RTF filter
-                             -------------------
-    begin                : 2006-07-03
-    copyright            : 2006 by CrossWire Bible Society
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+ *                    teirtf.cpp  -  TEI to RTF filter
+ *                            -------------------
+ *   begin                : 2006-07-03
+ *   copyright            : 2006 by CrossWire Bible Society
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
 
 #include <stdlib.h>
 #include <ctype.h>
@@ -26,6 +32,7 @@ SWORD_NAMESPACE_START
 
 TEIRTF::MyUserData::MyUserData(const SWModule *module, const SWKey *key) : BasicFilterUserData(module, key) {
 	BiblicalText = false;
+	inOsisRef = false;
 	if (module) {
 		version = module->Name();
 		BiblicalText = (!strcmp(module->Type(), "Biblical Texts"));
@@ -66,10 +73,10 @@ bool TEIRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *use
 		}
 
 		// <hi>
-		else if (!strcmp(tag.getName(), "hi")) {
+		else if (!strcmp(tag.getName(), "hi") || !strcmp(tag.getName(), "emph")) {
 			SWBuf rend = tag.getAttribute("rend");
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				if (rend == "ital")
+				if (rend == "ital" || rend == "italic")
 					buf += "{\\i1 ";
 				else if (rend == "bold")
 					buf += "{\\b1 ";
@@ -169,6 +176,25 @@ bool TEIRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *use
 			}
 		}
 
+		// <lb/> tag
+		else if (!strcmp(tag.getName(), "lb")) {
+			buf += "{\\par}";
+			userData->supressAdjacentWhitespace = true;
+		}
+
+		// <ref> tag
+		else if (!strcmp(tag.getName(), "ref")) {
+			if (!tag.isEndTag() && tag.getAttribute("osisRef")) {
+				buf += "{<a href=\"\">";
+				u->inOsisRef = true;
+			}
+			else if (tag.isEndTag() && u->inOsisRef) {
+				buf += "</a>}";
+				u->inOsisRef = false;
+			}
+		}
+
+
 		else {
 			return false;  // we still didn't handle token
 		}
