@@ -2,9 +2,24 @@
  *
  * thmlstrongs -	SWFilter descendant to hide or show strongs number
  *			in a ThML module.
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
  */
 
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -326,7 +341,7 @@ char ThMLOSIS::processText(SWBuf &text, const SWKey *key, const SWModule *module
 			}
 
 			// Footnote
-			if (!strcmp(token, "note")) {
+			if (!strncmp(token, "note", 4)) {
 		        	//pushString(&to, "<note>");
 				text.append("<note>");
 				newText = true;
@@ -534,23 +549,23 @@ char ThMLOSIS::processText(SWBuf &text, const SWKey *key, const SWModule *module
 			text = ref + text;
 			
 			if (vkey->Verse()) {
-				VerseKey tmp;
-				tmp = *vkey;
-				tmp.AutoNormalize(0);
-				tmp.Headings(1);
-				
+				VerseKey *tmp = (VerseKey *)vkey->clone();
+				*tmp = *vkey;
+				tmp->AutoNormalize(0);
+				tmp->Headings(1);
+
 				text += "</verse>";
-				
-				tmp = MAXVERSE;
-				if (*vkey == tmp) {
-					tmp.Verse(0);
+
+				*tmp = MAXVERSE;
+				if (*vkey == *tmp) {
+					tmp->Verse(0);
 //					sprintf(ref, "\t</div>");
 //					pushString(&to, ref);
-					tmp = MAXCHAPTER;
-					tmp = MAXVERSE;
-					if (*vkey == tmp) {
-						tmp.Chapter(0);
-						tmp.Verse(0);
+					*tmp = MAXCHAPTER;
+					*tmp = MAXVERSE;
+					if (*vkey == *tmp) {
+						tmp->Chapter(0);
+						tmp->Verse(0);
 //						sprintf(ref, "\t</div>");
 //						pushString(&to, ref);
 /*
@@ -561,6 +576,7 @@ char ThMLOSIS::processText(SWBuf &text, const SWKey *key, const SWModule *module
 */
 					}
 				}
+                                delete tmp;
 			}
 //			else if (vkey->Chapter()) {
 //				sprintf(ref, "\t<div type=\"chapter\" osisID=\"%s\">", vkey->getOSISRef());
