@@ -1,8 +1,24 @@
 /******************************************************************************
-*
-* utf8transliterators - SWFilter descendant to transliterate between
-*			 ICU-supported scripts.
-*/
+ *
+ * utf8transliterators - SWFilter descendant to transliterate between
+ *			 ICU-supported scripts.
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
 
 #ifdef _ICU_
 
@@ -25,51 +41,48 @@ SWORD_NAMESPACE_START
 const char UTF8Transliterator::optionstring[NUMTARGETSCRIPTS][16] = {
         "Off",
         "Latin",
-        "IPA",
-        "Basic Latin",
-        "SBL",
-        "TC",
-        "Beta",
-        "BGreek",
-        "SERA",
-        "Hugoye",
-        "UNGEGN",
-        "ISO",
-        "ALA-LC",
-        "BGN-PCGN",
-        "Greek",
-        "Hebrew",
-        "Cyrillic",
-        "Arabic",
-        "Syriac",
-        "Katakana",
-        "Hiragana",
-        "Hangul",
-        "Devanagari",
-        "Tamil",
-        "Bengali",
-        "Gurmukhi",
-        "Gujarati",
-        "Oriya",
-        "Telugu",
-        "Kannada",
-        "Malayalam",
-        "Thai",
-        "Georgian",
-        "Armenian",
-        "Ethiopic",
-        "Gothic",
-        "Ugaritic",
-        "Coptic",
-        "Meroitic",
-        "Linear B",
-        "Cypriot",
-        "Runic",
-        "Ogham",
-        "Thaana",
-	"Glagolitic",
-//        "Tengwar",
-//        "Cirth"
+	//        "IPA",
+	//        "Basic Latin",
+	//        "SBL",
+	//        "TC",
+	//        "Beta",
+	//        "BGreek",
+	//        "SERA",
+	//        "Hugoye",
+	//        "UNGEGN",
+	//        "ISO",
+	//        "ALA-LC",
+	//        "BGN",
+	//        "Greek",
+	//        "Hebrew",
+	//        "Cyrillic",
+	//        "Arabic",
+	//        "Syriac",
+	//        "Katakana",
+	//        "Hiragana",
+	//        "Hangul",
+	//        "Devanagari",
+	//        "Tamil",
+	//        "Bengali",
+	//        "Gurmukhi",
+	//        "Gujarati",
+	//        "Oriya",
+	//        "Telugu",
+	//        "Kannada",
+	//        "Malayalam",
+	//        "Thai",
+	//        "Georgian",
+	//        "Armenian",
+	//        "Ethiopic",
+	//        "Gothic",
+	//        "Ugaritic",
+	//        "Coptic",
+	//        "Linear B",
+	//        "Cypriot",
+	//        "Runic",
+	//        "Ogham",
+	//        "Thaana",
+	//        "Glagolitic",
 };
 
 const char UTF8Transliterator::optName[] = "Transliteration";
@@ -128,6 +141,11 @@ UTF8Transliterator::UTF8Transliterator() {
 #endif
 }
 
+
+UTF8Transliterator::~UTF8Transliterator() {
+}
+
+
 void UTF8Transliterator::Load(UErrorCode &status)
 {
 #ifndef _ICUSWORD_
@@ -374,19 +392,19 @@ char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModu
 			case UBLOCK_MALAYALAM: scripts[SE_MALAYALAM] = true; break;
 			case UBLOCK_THAI: scripts[SE_THAI] = true; break;
 			case UBLOCK_GEORGIAN: scripts[SE_GEORGIAN] = true; break;
-      			case UBLOCK_ARMENIAN: scripts[SE_ARMENIAN] = true; break;
-                        case UBLOCK_ETHIOPIC: scripts[SE_ETHIOPIC] = true; break;
-                        case UBLOCK_GOTHIC: scripts[SE_GOTHIC] = true; break;
-                        case UBLOCK_UGARITIC: scripts[SE_UGARITIC] = true; break;
-//                        case UBLOCK_MEROITIC: scripts[SE_MEROITIC] = true; break;
-//                        case UBLOCK_LINEARB: scripts[SE_LINEARB] = true; break;
-//                        case UBLOCK_CYPRIOT: scripts[SE_CYPRIOT] = true; break;
-                        case UBLOCK_RUNIC: scripts[SE_RUNIC] = true; break;
-                        case UBLOCK_OGHAM: scripts[SE_OGHAM] = true; break;
-                        case UBLOCK_THAANA: scripts[SE_THAANA] = true; break;
-//                        case UBLOCK_GLAGOLITIC: scripts[SE_GLAGOLITIC] = true; break;
-//                        case UBLOCK_TENGWAR: scripts[SE_TENGWAR] = true; break;
-//                        case UBLOCK_CIRTH: scripts[SE_CIRTH] = true; break;
+			case UBLOCK_ARMENIAN: scripts[SE_ARMENIAN] = true; break;
+			case UBLOCK_ETHIOPIC: scripts[SE_ETHIOPIC] = true; break;
+			case UBLOCK_GOTHIC: scripts[SE_GOTHIC] = true; break;
+			case UBLOCK_UGARITIC: scripts[SE_UGARITIC] = true; break;
+//			case UBLOCK_MEROITIC: scripts[SE_MEROITIC] = true; break;
+			case UBLOCK_LINEAR_B_SYLLABARY: scripts[SE_LINEARB] = true; break;
+			case UBLOCK_CYPRIOT_SYLLABARY: scripts[SE_CYPRIOT] = true; break;
+			case UBLOCK_RUNIC: scripts[SE_RUNIC] = true; break;
+			case UBLOCK_OGHAM: scripts[SE_OGHAM] = true; break;
+			case UBLOCK_THAANA: scripts[SE_THAANA] = true; break;
+			case UBLOCK_GLAGOLITIC: scripts[SE_GLAGOLITIC] = true; break;
+//			case UBLOCK_TENGWAR: scripts[SE_TENGWAR] = true; break;
+//			case UBLOCK_CIRTH: scripts[SE_CIRTH] = true; break;
 			case UBLOCK_CJK_RADICALS_SUPPLEMENT:
 			case UBLOCK_KANGXI_RADICALS:
 			case UBLOCK_IDEOGRAPHIC_DESCRIPTION_CHARACTERS:
@@ -445,8 +463,8 @@ char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModu
         	                	addTrans("Greek-Latin/ISO", &ID);
                                 else if (option == SE_ALALC)
         	                	addTrans("Greek-Latin/ALALC", &ID);
-                                else if (option == SE_BGNPCGN)
-        	                	addTrans("Greek-Latin/BGNPCGN", &ID);
+                                else if (option == SE_BGN)
+        	                	addTrans("Greek-Latin/BGN", &ID);
                                 else if (option == SE_IPA)
         	                	addTrans("Greek-IPA/Ancient", &ID);
                                 else {
@@ -522,8 +540,8 @@ char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModu
         			addTrans("Georgian-Latin/ISO", &ID);
                         else if (option == SE_ALALC)
         			addTrans("Georgian-Latin/ALALC", &ID);
-                        else if (option == SE_BGNPCGN)
-        			addTrans("Georgian-Latin/BGNPCGN", &ID);
+                        else if (option == SE_BGN)
+        			addTrans("Georgian-Latin/BGN", &ID);
                         else if (option == SE_IPA)
         			addTrans("Georgian-IPA", &ID);
                         else {
@@ -536,8 +554,8 @@ char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModu
         			addTrans("Armenian-Latin/ISO", &ID);
                         else if (option == SE_ALALC)
         			addTrans("Armenian-Latin/ALALC", &ID);
-                        else if (option == SE_BGNPCGN)
-        			addTrans("Armenian-Latin/BGNPCGN", &ID);
+                        else if (option == SE_BGN)
+        			addTrans("Armenian-Latin/BGN", &ID);
                         else if (option == SE_IPA)
         			addTrans("Armenian-IPA", &ID);
                         else {
@@ -600,8 +618,8 @@ char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModu
 		if (scripts[SE_THAANA]) {
 			if (option == SE_ALALC)
                         	addTrans("Thaana-Latin/ALALC", &ID);
-			else if (option == SE_BGNPCGN)
-                        	addTrans("Thaana-Latin/BGNPCGN", &ID);
+			else if (option == SE_BGN)
+                        	addTrans("Thaana-Latin/BGN", &ID);
 			else {
 				addTrans("Thaana-Latin", &ID);
 				scripts[SE_LATIN] = true;
