@@ -1,11 +1,29 @@
 /******************************************************************************
- *  swmodule.cpp -code for base class 'module'.  Module is the basis for all
- *		  types of modules (e.g. texts, commentaries, maps, lexicons,
- *		  etc.)
+ *  swmodule.cpp - code for base class 'module'.  Module is the basis for all
+ *		   types of modules (e.g. texts, commentaries, maps, lexicons,
+ *		   etc.)
+ *
+ *
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
  */
 
+
 #include <vector>
 
+#include <swlog.h>
 #include <sysdata.h>
 #include <swmodule.h>
 #include <utilstr.h>
@@ -58,7 +76,7 @@ typedef std::list<SWBuf> StringList;
  *	unicode  - if this module is unicode
  */
 
-SWModule::SWModule(const char *imodname, const char *imoddesc, SWDisplay *idisp, char *imodtype, SWTextEncoding encoding, SWTextDirection direction, SWTextMarkup markup, const char* imodlang) {
+SWModule::SWModule(const char *imodname, const char *imoddesc, SWDisplay *idisp, const char *imodtype, SWTextEncoding encoding, SWTextDirection direction, SWTextMarkup markup, const char *imodlang) {
 	key       = CreateKey();
 	entryBuf  = "";
 	config    = &ownConfig;
@@ -127,7 +145,7 @@ SWModule::~SWModule()
  * RET:	pointer to allocated key
  */
 
-SWKey *SWModule::CreateKey()
+SWKey *SWModule::CreateKey() const
 {
 	return new SWKey();
 }
@@ -255,7 +273,8 @@ char SWModule::Markup(signed char newmark) {
 
 char *SWModule::Lang(const char *imodlang)
 {
-	return stdstr(&modlang, imodlang);
+	if (imodlang) stdstr(&modlang, imodlang);
+	return modlang;
 }
 
 
@@ -395,8 +414,9 @@ void SWModule::decrement(int steps) {
  *				>=0 - regex
  *				-1  - phrase
  *				-2  - multiword
- *				-3  - entryAttrib (eg. Word//Lemma/G1234/)
+ *				-3  - entryAttrib (eg. Word//Lemma./G1234/)	 (Lemma with dot means check components (Lemma.[1-9]) also)
  *				-4  - clucene
+ *				-5  - multilemma window; flags = window size
  * 	flags		- options flags for search
  *	justCheckIfSupported	- if set, don't search, only tell if this
  *							function supports requested search.
@@ -408,6 +428,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 
 	listKey.ClearList();
 	SWBuf term = istr;
+	bool includeComponents = false;	// for entryAttrib e.g., /Lemma.1/ 
 
 #ifdef USELUCENE
 	SWBuf target = getConfigEntry("AbsoluteDataPath");
@@ -431,12 +452,16 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 	SWKey *resultKey = CreateKey();
 	regex_t preg;
 	vector<SWBuf> words;
+	vector<SWBuf> window;
 	const char *sres;
 	terminateSearch = false;
 	char perc = 1;
 	bool savePEA = isProcessEntryAttributes();
-   // determine if we might be doing special strip searches.  useful for knowing if we can use shortcuts
-	bool specialStrips = (getConfigEntry("LocalStripFilter") || strchr(istr, '<'));
+
+	// determine if we might be doing special strip searches.  useful for knowing if we can use shortcuts
+	bool specialStrips = (getConfigEntry("LocalStripFilter")
+                       || (getConfig().has("GlobalOptionFilter", "UTF8GreekAccents"))
+                       || (strchr(istr, '<')));
 
 	processEntryAttributes(searchType == -3);
 	
@@ -454,19 +479,9 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 	}
 
 	(*percent)(perc, percentUserData);
-	// MAJOR KLUDGE: VerseKey::Index still return index within testament.
-	// 	VerseKey::NewIndex should be moved to Index and Index should be some
-	// 	VerseKey specific name
-	VerseKey *vkcheck = 0;
-	SWTRY {
-		vkcheck = SWDYNAMIC_CAST(VerseKey, key);
-	}
-	SWCATCH (...) {}
-	// end MAJOR KLUDGE
 
 	*this = BOTTOM;
-	// fix below when we find out the bug
-	long highIndex = (vkcheck)?32300/*vkcheck->NewIndex()*/:key->Index();
+	long highIndex = key->Index();
 	if (!highIndex)
 		highIndex = 1;		// avoid division by zero errors.
 	*this = TOP;
@@ -502,6 +517,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 			(*percent)(80, percentUserData);
 
 			// iterate thru each good module position that meets the search
+			bool checkBounds = getKey()->isBoundSet();
 			for (long i = 0; i < h->length(); i++) {
 				Document &doc = h->doc(i);
 
@@ -509,12 +525,15 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 				lucene_wcstoutf8(utfBuffer, doc.get(_T("key")), MAX_CONV_SIZE);	
 				*resultKey = utfBuffer; //TODO Does a key always accept utf8?
 
-				// check to see if it sets ok (in our range?) and if so, add to our return list
-				*getKey() = *resultKey;
-				if (*getKey() == *resultKey) {
-					listKey << *resultKey;
-					listKey.GetElement()->userData = (void *)((__u32)(h->score(i)*100));
+				// check to see if it sets ok (within our bounds) and if not, skip
+				if (checkBounds) {
+					*getKey() = *resultKey;
+					if (*getKey() != *resultKey) {
+						continue;
+					}
 				}
+				listKey << *resultKey;
+				listKey.GetElement()->userData = (void *)((__u32)(h->score(i)*100));
 			}
 			(*percent)(98, percentUserData);
 		}
@@ -543,6 +562,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 
 	// multi-word
 	case -2:
+	case -5:
 		// let's break the term down into our words vector
 		while (1) {
 			const char *word = term.stripPrefix(' ');
@@ -570,6 +590,10 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 			}
 			words.push_back(word);
 		}
+		if ((words.size()>2) && words[2].endsWith(".")) {
+			includeComponents = true;
+			words[2]--;
+		}
 		break;
 	}
 
@@ -579,16 +603,12 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 	(*percent)(perc, percentUserData);
 
 	
-	while ((searchType > -4) && !Error() && !terminateSearch) {
-		long mindex = 0;
-		if (vkcheck)
-			mindex = vkcheck->NewIndex();
-		else mindex = key->Index();
+	while ((searchType != -4) && !Error() && !terminateSearch) {
+		long mindex = key->Index();
 		float per = (float)mindex / highIndex;
 		per *= 93;
 		per += 5;
 		char newperc = (char)per;
-//		char newperc = (char)(5+(93*(((float)((vkcheck)?vkcheck->NewIndex():key->Index()))/highIndex)));
 		if (newperc > perc) {
 			perc = newperc;
 			(*percent)(perc, percentUserData);
@@ -596,8 +616,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 		else if (newperc < perc) {
 #ifndef _MSC_VER
 			std::cerr << "Serious error: new percentage complete is less than previous value\n";
-			std::cerr << "using vk? " << ((vkcheck)?"yes":"no") << "\n";
-			std::cerr << "index: " << ((vkcheck)?vkcheck->NewIndex():key->Index()) << "\n";
+			std::cerr << "index: " << (key->Index()) << "\n";
 			std::cerr << "highIndex: " << highIndex << "\n";
 			std::cerr << "newperc ==" << (int)newperc << "%" << "is smaller than\n";
 			std::cerr << "perc == "  << (int )perc << "% \n";
@@ -656,7 +675,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 				} break;
 
 			// entry attributes
-			case -3:
+			case -3: {
 				RenderText();	// force parse
 				AttributeTypeList &entryAttribs = getEntryAttributes();
 				AttributeTypeList::iterator i1Start, i1End;
@@ -685,7 +704,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 						i2End   = i1Start->second.end();
 					}
 					for (;i2Start != i2End; i2Start++) {
-						if ((words.size()>2) && (words[2].length())) {
+						if ((words.size()>2) && (words[2].length()) && (!includeComponents)) {
 							i3Start = i2Start->second.find(words[2]);
 							i3End = i3Start;
 							if (i3End != i2Start->second.end())
@@ -697,6 +716,13 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 						}
 						for (;i3Start != i3End; i3Start++) {
 							if ((words.size()>3) && (words[3].length())) {
+								if (includeComponents) {
+									SWBuf key = i3Start->first.c_str();
+									key = key.stripPrefix('.', true);
+									// we're iterating all 3 level keys, so be sure we match our
+									// prefix (e.g., Lemma, Lemma.1, Lemma.2, etc.)
+									if (key != words[2]) continue;
+								}
 								if (flags & SEARCHFLAG_MATCHWHOLEENTRY) {
 									bool found = !(((flags & REG_ICASE) == REG_ICASE) ? sword::stricmp(i3Start->second.c_str(), words[3]) : strcmp(i3Start->second.c_str(), words[3]));
 									sres = (found) ? i3Start->second.c_str() : 0;
@@ -719,6 +745,54 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 						break;
 				}
 				break;
+			}
+			case -5:
+				AttributeList &words = getEntryAttributes()["Word"];
+				SWBuf kjvWord = "";
+				SWBuf bibWord = "";
+				for (AttributeList::iterator it = words.begin(); it != words.end(); it++) {
+					int parts = atoi(it->second["PartCount"]);
+					SWBuf lemma = "";
+					SWBuf morph = "";
+					for (int i = 1; i <= parts; i++) {
+						SWBuf key = "";
+						key = (parts == 1) ? "Lemma" : SWBuf().setFormatted("Lemma.%d", i).c_str();
+						AttributeValue::iterator li = it->second.find(key);
+						if (li != it->second.end()) {
+							if (i > 1) lemma += " ";
+							key = (parts == 1) ? "LemmaClass" : SWBuf().setFormatted("LemmaClass.%d", i).c_str();
+							AttributeValue::iterator lci = it->second.find(key);
+							if (lci != it->second.end()) {
+								lemma += lci->second + ":";
+							}
+							lemma += li->second;
+						}
+						key = (parts == 1) ? "Morph" : SWBuf().setFormatted("Morph.%d", i).c_str();
+						li = it->second.find(key);
+						// silly.  sometimes morph counts don't equal lemma counts
+						if (i == 1 && parts != 1 && li == it->second.end()) {
+							li = it->second.find("Morph");
+						}
+						if (li != it->second.end()) {
+							if (i > 1) morph += " ";
+							key = (parts == 1) ? "MorphClass" : SWBuf().setFormatted("MorphClass.%d", i).c_str();
+							AttributeValue::iterator lci = it->second.find(key);
+							// silly.  sometimes morph counts don't equal lemma counts
+							if (i == 1 && parts != 1 && lci == it->second.end()) {
+								lci = it->second.find("MorphClass");
+							}
+							if (lci != it->second.end()) {
+								morph += lci->second + ":";
+							}
+							morph += li->second;
+						}
+						// TODO: add src tags and maybe other attributes
+					}
+					while (window.size() < (unsigned)flags) {
+						
+					}
+				}
+				break;
 			} // end switch
 		}
 		(*this)++;
@@ -812,7 +886,7 @@ const char *SWModule::StripText(const char *buf, int len) {
  * RET: this module's text at current key location massaged by RenderFilers
  */
 
- const char *SWModule::RenderText(SWKey *tmpKey) {
+ const char *SWModule::RenderText(const SWKey *tmpKey) {
 	SWKey *saveKey;
 	const char *retVal;
 
@@ -843,7 +917,7 @@ const char *SWModule::StripText(const char *buf, int len) {
  * RET: this module's text at specified key location massaged by Strip filters
  */
 
-const char *SWModule::StripText(SWKey *tmpKey) {
+const char *SWModule::StripText(const SWKey *tmpKey) {
 	SWKey *saveKey;
 	const char *retVal;
 
@@ -919,8 +993,8 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		}
 	}
 
-	
-	// be sure we give CLucene enough file handles	
+
+	// be sure we give CLucene enough file handles
 	FileMgr::getSystemFileMgr()->flush();
 
 	// save key information so as not to disrupt original
@@ -941,7 +1015,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 	IndexWriter *coreWriter = NULL;
 	IndexWriter *fsWriter = NULL;
 	Directory *d = NULL;
- 
+
 	standard::StandardAnalyzer *an = new standard::StandardAnalyzer();
 	SWBuf target = getConfigEntry("AbsoluteDataPath");
 	bool includeKeyInSearch = getConfig().has("SearchOption", "IncludeKeyInSearch");
@@ -953,19 +1027,21 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 
 	ramDir = new RAMDirectory();
 	coreWriter = new IndexWriter(ramDir, an, true);
-	
 
- 
+
+
 	char perc = 1;
 	VerseKey *vkcheck = 0;
 	vkcheck = SWDYNAMIC_CAST(VerseKey, key);
+	VerseKey *chapMax = 0;
+        if (vkcheck) chapMax = (VerseKey *)vkcheck->clone();
 
 	TreeKeyIdx *tkcheck = 0;
 	tkcheck = SWDYNAMIC_CAST(TreeKeyIdx, key);
 
 
 	*this = BOTTOM;
-	long highIndex = (vkcheck)?32300/*vkcheck->NewIndex()*/:key->Index();
+	long highIndex = key->Index();
 	if (!highIndex)
 		highIndex = 1;		// avoid division by zero errors.
 
@@ -976,20 +1052,16 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 	// position module at the beginning
 	*this = TOP;
 
-	VerseKey chapMax;
 	SWBuf proxBuf;
 	SWBuf proxLem;
 	SWBuf strong;
 
 	const short int MAX_CONV_SIZE = 2047;
 	wchar_t wcharBuffer[MAX_CONV_SIZE + 1];
-	
+
 	char err = Error();
 	while (!err) {
-		long mindex = 0;
-		if (vkcheck)
-			mindex = vkcheck->NewIndex();
-		else mindex = key->Index();
+		long mindex = key->Index();
 
 		proxBuf = "";
 		proxLem = "";
@@ -1016,7 +1088,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		if (content && *content) {
 			good = true;
 
-			
+
 			// build "strong" field
 			AttributeTypeList::iterator words;
 			AttributeList::iterator word;
@@ -1046,8 +1118,10 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 			}
 
 			lucene_utf8towcs(wcharBuffer, keyText, MAX_CONV_SIZE); //keyText must be utf8
+//			doc->add( *(new Field("key", wcharBuffer, Field::STORE_YES | Field::INDEX_TOKENIZED)));
 			doc->add( *Field::Text(_T("key"), wcharBuffer ) );
 
+
 			if (includeKeyInSearch) {
 				c = keyText;
 				c += " ";
@@ -1071,12 +1145,12 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 
 		// for VerseKeys use chapter
 		if (vkcheck) {
-			chapMax = *vkcheck;
+			*chapMax = *vkcheck;
 			// we're the first verse in a chapter
 			if (vkcheck->Verse() == 1) {
-				chapMax = MAXVERSE;
+				*chapMax = MAXVERSE;
 				VerseKey saveKey = *vkcheck;
-				while ((!err) && (*vkcheck <= chapMax)) {
+				while ((!err) && (*vkcheck <= *chapMax)) {
 //printf("building proxBuf from (%s).\nproxBuf.c_str(): %s\n", (const char *)*key, proxBuf.c_str());
 //printf("building proxBuf from (%s).\n", (const char *)*key);
 
@@ -1112,7 +1186,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 						proxBuf += content;
 						proxBuf.append(' ');
 						proxLem += strong;
-						if (proxLem.length()) 
+						if (proxLem.length())
 							proxLem.append("\n");
 					}
 					(*this)++;
@@ -1122,7 +1196,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 				*vkcheck = saveKey;
 			}
 		}
-		
+
 		// for TreeKeys use siblings if we have no children
 		else if (tkcheck) {
 			if (!tkcheck->hasChildren()) {
@@ -1164,7 +1238,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 							proxBuf += content;
 							proxBuf.append(' ');
 							proxLem += strong;
-							if (proxLem.length()) 
+							if (proxLem.length())
 								proxLem.append("\n");
 						}
 					} while (tkcheck->nextSibling());
@@ -1174,11 +1248,11 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 				else tkcheck->nextSibling();	// reposition from our previousSibling test
 			}
 		}
-		
+
 		if (proxBuf.length() > 0) {
-	
+
 			lucene_utf8towcs(wcharBuffer, proxBuf, MAX_CONV_SIZE); //keyText must be utf8
-		
+
 //printf("proxBuf after (%s).\nprox: %s\nproxLem: %s\n", (const char *)*key, proxBuf.c_str(), proxLem.c_str());
 
 			doc->add( *Field::UnStored(_T("prox"), wcharBuffer) );
@@ -1209,7 +1283,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		if (IndexReader::isLocked(d)) {
 			IndexReader::unlock(d);
 		}
- 
+
 		fsWriter = new IndexWriter( d, an, false);
 	} else {
 		d = FSDirectory::getDirectory(target.c_str(), true);
@@ -1234,6 +1308,8 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 	if (searchKey)
 		delete searchKey;
 
+        delete chapMax;
+
 	processEntryAttributes(savePEA);
 
 	// reset option filters back to original values
@@ -1253,7 +1329,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
  * @param buf the buffer to filter
  * @param key key location from where this buffer was extracted
  */
-void SWModule::filterBuffer(OptionFilterList *filters, SWBuf &buf, SWKey *key) {
+void SWModule::filterBuffer(OptionFilterList *filters, SWBuf &buf, const SWKey *key) {
 	OptionFilterList::iterator it;
 	for (it = filters->begin(); it != filters->end(); it++) {
 		(*it)->processText(buf, key, this);
@@ -1265,7 +1341,7 @@ void SWModule::filterBuffer(OptionFilterList *filters, SWBuf &buf, SWKey *key) {
  * @param buf the buffer to filter
  * @param key key location from where this buffer was extracted
  */
-void SWModule::filterBuffer(FilterList *filters, SWBuf &buf, SWKey *key) {
+void SWModule::filterBuffer(FilterList *filters, SWBuf &buf, const SWKey *key) {
 	FilterList::iterator it;
 	for (it = filters->begin(); it != filters->end(); it++) {
 		(*it)->processText(buf, key, this);
@@ -1282,4 +1358,63 @@ void SWModule::setEntry(const char*, long) {
 void SWModule::linkEntry(const SWKey*) {
 }
 
+
+/******************************************************************************
+ * SWModule::prepText	- Prepares the text before returning it to external
+ *					objects
+ *
+ * ENT:	buf	- buffer where text is stored and where to store the prep'd
+ *				text.
+ */
+
+void SWModule::prepText(SWBuf &buf) {
+	unsigned int to, from; 
+	char space = 0, cr = 0, realdata = 0, nlcnt = 0;
+	char *rawBuf = buf.getRawData();
+	for (to = from = 0; rawBuf[from]; from++) {
+		switch (rawBuf[from]) {
+		case 10:
+			if (!realdata)
+				continue;
+			space = (cr) ? 0 : 1;
+			cr = 0;
+			nlcnt++;
+			if (nlcnt > 1) {
+//				*to++ = nl;
+				rawBuf[to++] = 10;
+//				*to++ = nl[1];
+//				nlcnt = 0;
+			}
+			continue;
+		case 13:
+			if (!realdata)
+				continue;
+//			*to++ = nl[0];
+			rawBuf[to++] = 10;
+			space = 0;
+			cr = 1;
+			continue;
+		}
+		realdata = 1;
+		nlcnt = 0;
+		if (space) {
+			space = 0;
+			if (rawBuf[from] != ' ') {
+				rawBuf[to++] = ' ';
+				from--;
+				continue;
+			}
+		}
+		rawBuf[to++] = rawBuf[from];
+	}
+	buf.setSize(to);
+
+	while (to > 1) {			// remove trailing excess
+		to--;
+		if ((rawBuf[to] == 10) || (rawBuf[to] == ' '))
+			buf.setSize(to);
+		else break;
+	}
+}
+
 SWORD_NAMESPACE_END
