@@ -1,7 +1,23 @@
 // Diatheke 4.2 by Chris Little <chrislit@crosswire.org>
-// Copyright 1999-2002 by CrossWire Bible Society http://www.crosswire.org
-// Licensed under GNU General Public License (GPL)
-// see accompanying LICENSE file for license details
+// Copyright 1999-2009 by CrossWire Bible Society
+// http://www.crosswire.org/sword/diatheke
+
+/*
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
 
 #include <stdio.h>
 #include <iostream>
@@ -31,7 +47,7 @@
 #define OP_BIDI 2048
 #define OP_VARIANTS 4096
 #define OP_RED 8192
-
+#define OP_ARABICPOINTS 16384
 #define ST_NONE 0
 #define ST_REGEX 1 //0
 #define ST_PHRASE 2  // -1
