@@ -1,14 +1,27 @@
 // Diatheke 4.2.1 by Chris Little <chrislit@crosswire.org>
 // Copyright 1999-2005 by CrossWire Bible Society
 // http://www.crosswire.org/sword/diatheke
-// Licensed under GNU General Public License (GPL)
-// see accompanying LICENSE file for license details
 
+/*
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
 
 #include "corediatheke.h"
 #include "diathekemgr.h"
 #include "diafiltmgr.h"
-#include <iostream>
 #include <utilstr.h>
 
 using std::cout;
@@ -33,7 +46,7 @@ void printsyntax() {
 	fprintf (stderr, "Valid search_type values are: regex, multiword, and phrase(def).\n");
 	fprintf (stderr, "Valid option_filters values are: n (Strong's numbers),\n");
 	fprintf (stderr, "  f (Footnotes), m (Morphology), h (Section Headings),\n");
-	fprintf (stderr, "  c (Cantillation), v (Hebrew Vowels), a (Greek Accents),\n");
+	fprintf (stderr, "  c (Cantillation), v (Hebrew Vowels), a (Greek Accents), p (Arabic Vowels)\n");
 	fprintf (stderr, "  l (Lemmas), s (Scripture Crossrefs), r (Arabic Shaping,\n");
 	fprintf (stderr, "  b (Bi-Directional Reordering), x (Red Words of Christ)\n");
 
@@ -130,6 +143,8 @@ int main(int argc, char **argv)
 					optionfilters |= OP_BIDI;
 				if (strchr(argv[i+1], 'x'))
 					optionfilters |= OP_RED;
+				if (strchr(argv[i+1], 'p'))
+					optionfilters |= OP_ARABICPOINTS;
 				i++;
 			}
 		}
