@@ -1,3 +1,20 @@
+/*
+ * Copyright 2009 CrossWire Bible Society (http://www.crosswire.org)
+ *	CrossWire Bible Society
+ *	P. O. Box 2528
+ *	Tempe, AZ  85280-2528
+ *
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation version 2.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ */
+
 #include <stdio.h>
 #include <rawtext.h>
 #include <swmgr.h>
@@ -33,15 +50,13 @@ void percentUpdate(char percent, void *userData) {
 
 int main(int argc, char **argv)
 {
-
 	if (argc != 2) {
 		fprintf(stderr, "usage: %s <modname>\n", argv[0]);
 		exit(-1);
 	}
-	
+
 	SWModule *target;
 	ListKey listkey;
-	VerseKey vk;
 	ModMap::iterator it;
 
 	SWMgr manager;
